# Project Description

Orchestrator manages platform for running docker containers, maintains database and provide rest API for the gui part of Analyza project.
On top of the native platform runtime, it also manages custom container dependencies and custom running states of containers.

This repository also contains:
 * __Desifintion Designer CLI__ - management of definition repository, simple parsing of definitions written in json format. 
 * __Orchestrator Client CLI__
 
# Documentation
 
* [Definitions of analytical operations](./docs/definition_creation.md)   
* [Environment setup](./docs/prepare_env.md)   
* [Work with environment](./docs/env_introduction.md)
* [API description](./docs/api-doc/swagger.yaml)

# File Structure

```Bash
   ├── client           # command line interface of client for orchestrator
   ├── designer         # command line interface to manage definitions in repository
   ├── server           # main of orchestrator
   ├── config  
   │   └── config.yaml  # default configuration file 
   ├── pkg              # various helper libraries
   ├── internal  
   │   ├── app          # core functions
   │   ├── config       # config parser
   │   ├── logger       # logging
   │   ├── model        # database model structures
   │   ├── repository   # drivers for metadata and definition repositories
   │   └── platform     # drivers for platform manager
   ├── test
   │   ├── examples     # example json files of definitions and requests
   │   └── dockerfiles  # docker images for test modules
   ├── docs             # project documentation e.g. API endpoints description 
   ├── README.md
   ├── kube.yaml        # kubernetes manifest for orchestrator image
   └── Dockerfile       # used for building docker image of orchestrator
```

# Build, Run and Configuration

## orchestrator versions

There are 2 versions of orchestrator:
* old design: branch "old-design"
* new design: branch "master"

## build requirements:

* [Golang](https://golang.org/doc/install) version >= 1.14
* git

## build and run instructions:

```bash
git clone gitlab.ics.muni.cz/ANALYZA/orchestrator
cd orchestrator
go build -o orchestrator ./server
./orchestrator
```

## configuration and start

### requirements;

Server that would be running orchestrator must have access to:
* running kubernetes cluster
* postgreSQL database

> __Note:__ To perform operations, all module docker images put in the orchestrator database must be pullable by all kubernetes nodes. One should put all modules into [local](https://docs.docker.com/registry/deploying/) docker registry.

To get orchestrator working, one must prepare configuration directory `config` which must contain:

* `config.yaml` file with correct values
* kubeconfig file used for access kubernetes

### start in docker

build docker image of orchestrator

```bash
docker build . -t orchestrator
```

populate configuration directory 

```bash
cp kube.config config/
vim config/config.yaml
```

start orchestrator container in background

```bash
docker run -s -p 8080:8080 --volume=config:/root/config orchestrator
```

# Definition Designer CLI

Build:
```bash
cd <orchestrator source dir>
go build -o orch-designer ./designer
```

Run:
```bash
./orch-designer --help
```
> __Note:__ designer also need to have config created and located in correct place. However, only connection to definition repository is needed to be configured.

# Client CLI

Build:
```bash
cd <orchestrator source dir>
go build -o orch-client ./client
```

Run:
```bash
./orch-client --help
```