// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package orchectl

import (
	"fmt"
	"github.com/spf13/cobra"
	"io/ioutil"
	"net/http"
)

var getCmd = &cobra.Command{
	Use:   "get",
	Short: "Print orchestrator object",
}

func getObj(urlPrefix, obj, id string) error {
	resp, err := http.Get(urlPrefix + obj + "/" + id)
	if err != nil {
		return err
	}
	body, err := ioutil.ReadAll(resp.Body)
	_ = resp.Body.Close()
	if err != nil {
		return err
	}
	fmt.Println(string(body))
	return nil
}

var getOperationCmd = &cobra.Command{
	Use:   "operation",
	Short: "Print running operation.",
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/", c.Address)
		if len(args) == 0 {
			err := getObj(urlPrefix, "operation", "")
			if err != nil {
				fmt.Println(err)
			}
		} else {
			for _, id := range args {
				err := getObj(urlPrefix, "operation", id)
				if err != nil {
					fmt.Println(err)
				}
			}
		}
		return nil
	},
}

var getNodeCmd = &cobra.Command{
	Use:   "node",
	Short: "Print node of running operation",
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/", c.Address)
		if len(args) == 0 {
			err := getObj(urlPrefix, "node", "")
			if err != nil {
				fmt.Println(err)
			}
		} else {
			for _, id := range args {
				err := getObj(urlPrefix, "node", id)
				if err != nil {
					fmt.Println(err)
				}
			}
		}
		return nil
	},
}

var getModuleCmd = &cobra.Command{
	Use:   "module",
	Short: "Print running module",
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/", c.Address)
		if len(args) == 0 {
			err := getObj(urlPrefix, "module", "")
			if err != nil {
				fmt.Println(err)
			}
		} else {
			for _, id := range args {
				err := getObj(urlPrefix, "module", id)
				if err != nil {
					fmt.Println(err)
				}
			}
		}
		return nil
	},
}

var getDefinitionCmd = &cobra.Command{
	Use:   "definition",
	Short: "Print definition.",
}

var getDefinitionModuleCmd = &cobra.Command{
	Use:   "module",
	Short: "Print definition of module.",
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/definition/", c.Address)
		if len(args) == 0 {
			err := getObj(urlPrefix, "module", "")
			if err != nil {
				fmt.Println(err)
			}
		} else {
			for _, id := range args {
				err := getObj(urlPrefix, "module", id)
				if err != nil {
					fmt.Println(err)
				}
			}
		}
		return nil
	},
}

var getDefinitionNodeCmd = &cobra.Command{
	Use:   "node",
	Short: "Print definition of operation node.",
	Args: cobra.MinimumNArgs(1),
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/definition/", c.Address)
		for _, id := range args {
			err := getObj(urlPrefix, "node", id)
			if err != nil {
				fmt.Println(err)
			}
		}
		return nil
	},
}

var getDefinitionOperationCmd = &cobra.Command{
	Use:   "operation",
	Short: "Print definition of operation.",
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		urlPrefix := fmt.Sprintf("http://%s/v1/definition/", c.Address)
		if len(args) == 0 {
			err := getObj(urlPrefix, "operation", "")
			if err != nil {
				fmt.Println(err)
			}
		} else {
			for _, id := range args {
				err := getObj(urlPrefix, "operation", id)
				if err != nil {
					fmt.Println(err)
				}
			}
		}
		return nil
	},
}