// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package orchectl

import (
	"context"
	"github.com/spf13/cobra"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/config"
	"os"
)

func Execute() {
	var configPath string
	rootCmd.PersistentFlags().StringVarP(&configPath, "config", "c","","path to configuration file")
	rootCmd.PersistentFlags().StringVarP(&configPath, "endpoint", "e","","address to orchestrator api")
	rootCmd.AddCommand(versionCmd, startCmd, stopCmd, getCmd)
	getCmd.AddCommand(getDefinitionCmd, getOperationCmd, getNodeCmd, getModuleCmd)
	getDefinitionCmd.AddCommand(getDefinitionNodeCmd, getDefinitionOperationCmd, getDefinitionModuleCmd)
	ctx := &DesignerCtx{
		Context:    context.Background(),
		ConfigPath: configPath,
	}
	if err := rootCmd.ExecuteContext(ctx); err != nil {
		os.Exit(1)
	}
}

var rootCmd = &cobra.Command{
	Use:   "orchectl",
	Short: "orchectl is a simple client for orchestrator",
	PersistentPreRunE: GetStoreFunc,
}

// type holds the definition store which can be used in any cobra command
type DesignerCtx struct {
	context.Context
	ConfigPath string
	Address string
}

var GetStoreFunc = func(cmd *cobra.Command, args []string) error {
	c, _ := cmd.Context().(*DesignerCtx)
	addr, _ := cmd.Flags().GetString("endpoint")
	conf, err := config.GetConfig(c.ConfigPath)
	if err == nil {
		c.Address = conf.Api.Address + ":" + conf.Api.Port
	}
	if addr != "" {
		c.Address = addr
	}
	return nil
}