// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package orchectl

import (
	"bytes"
	"encoding/json"
	"fmt"
	"github.com/spf13/cobra"
	"io/ioutil"
	"net/http"
	"os"
)

var startCmd = &cobra.Command{
	Use:   "start",
	Short: "Start new operation",
	Args: cobra.MinimumNArgs(1),
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		type operationCreate struct {
			ID string
			Name string
			Parameters map[string]map[string]string
		}
		type request struct {
			Requests []operationCreate
		}
		for _, file := range args {
			f, err := os.Open(file)
			if err != nil {
				fmt.Println(err)
				continue
			}
			data, err := ioutil.ReadAll(f)
			f.Close()
			if err != nil {
				fmt.Println(err)
				continue
			}
			var request request
			err = json.Unmarshal(data, &request)
			if err != nil {
				fmt.Println(err)
				continue
			}
			for _, r := range request.Requests {
				req, err := json.Marshal(&r)
				if err != nil {
					fmt.Println(err)
					continue
				}
				resp, err := http.Post("http://" + c.Address + "/v1/operation", "application/json", bytes.NewBuffer(req))
				if err != nil {
					return err
				}
				// Read Response Body
				respBody, err := ioutil.ReadAll(resp.Body)
				_ = resp.Body.Close()
				if err != nil {
					return err
				}
				fmt.Println(string(respBody))
			}
		}
		return nil
	},
}
