// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package orchectl

import (
	"fmt"
	"github.com/spf13/cobra"
	"io/ioutil"
	"net/http"
)

func deleteRequest(prefix, obj, id string) error {
	// Create client
	client := &http.Client{}
	// Create request
	req, err := http.NewRequest("DELETE", prefix + obj + "/" + id, nil)
	if err != nil {
		return err
	}
	// Fetch Request
	resp, err := client.Do(req)
	if err != nil {
		return err
	}
	// Read Response Body
	respBody, err := ioutil.ReadAll(resp.Body)
	_ = resp.Body.Close()
	if err != nil {
		return err
	}
	fmt.Println(string(respBody))
	return nil
}

var stopCmd = &cobra.Command{
	Use:   "stop",
	Short: "stop running operation",
	Args: cobra.MinimumNArgs(1),
	RunE: func(cmd *cobra.Command, args []string) error {
		// fetch context with connection to orchestrator
		c, _ := cmd.Context().(*DesignerCtx)
		prefix := fmt.Sprintf("http://%s/v1/", c.Address)
		for _, id := range args {
			err := deleteRequest(prefix, "operation", id)
			if err != nil {
				fmt.Println(err)
			}
		}
		return nil
	},
}
