// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cmd

import (
	"fmt"
	"github.com/spf13/cobra"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
	"strconv"
)

func init() {
	deleteCmd.AddCommand(deleteOperationCmd)
	deleteCmd.AddCommand(deleteNodeCmd)
	deleteCmd.AddCommand(deleteModuleCmd)
}

var deleteCmd = &cobra.Command{
	Use:   "delete",
	Short: "Delete definitions",
	Long:  "Command Delete removes definition of chosen objects from repository.",
}

var deleteOperationCmd = &cobra.Command{
	Use:   "operation",
	Short: "Delete definition of operation",
	Args: cobra.MinimumNArgs(1),
	RunE:  func(cmd *cobra.Command, args []string) error {
		// get access to definition store
		c, _ := cmd.Context().(*DesignerCtx)
		for _, name := range args {
			err := c.Storer.DeleteOperation(&model.OperationDefinition{Name: name})
			if err != nil {
				fmt.Printf("error removing %s from repository: %s", name, err)
			}
		}
		return nil
	},
}

var deleteNodeCmd = &cobra.Command{
	Use:   "node",
	Short: "Delete definition of node",
	Args: cobra.MinimumNArgs(1),
	RunE:  func(cmd *cobra.Command, args []string) error {
		// get access to definition store
		c, _ := cmd.Context().(*DesignerCtx)
		for _, name := range args {
			id, err := strconv.Atoi(name)
			if err != nil || uint(id) == 0 {
				continue
			}
			err = c.Storer.DeleteNode(&model.NodeDefinition{ID: uint(id)})
			if err != nil {
				fmt.Printf("error removing %s from repository: %s", name, err)
			}
			err = c.Storer.DeleteEdge(&model.EdgeDefinition{FromNode: uint(id)})
			if err != nil {
				fmt.Printf("error removing edge of %s from repository: %s", name, err)
			}
			err = c.Storer.DeleteEdge(&model.EdgeDefinition{ToNode: uint(id)})
			if err != nil {
				fmt.Printf("error removing edge of %s from repository: %s", name, err)
			}
		}
		return nil
	},
}

var deleteModuleCmd = &cobra.Command{
	Use:   "module",
	Short: "Delete definition of module",
	Args: cobra.MinimumNArgs(1),
	RunE:  func(cmd *cobra.Command, args []string) error {
		// get access to definition store
		c, _ := cmd.Context().(*DesignerCtx)
		for _, name := range args {
			err := c.Storer.DeleteModule(&model.ModuleDefinition{Name: name})
			if err != nil {
				fmt.Printf("error removing %s from repository: %s", name, err)
			}
		}
		return nil
	},
}
