// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cmd

import (
	"context"
	"github.com/pkg/errors"
	"github.com/spf13/cobra"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/config"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/repository"
	"os"
)

func Execute() {
	var configPath string
	rootCmd.PersistentFlags().StringVarP(&configPath, "config", "c","","path to configuration file (required)")
	rootCmd.AddCommand(versionCmd)
	rootCmd.AddCommand(deleteCmd)
	rootCmd.AddCommand(createCmd)
	//rootCmd.AddCommand(validateCmd)
	rootCmd.AddCommand(getCmd)

	ctx := &DesignerCtx{
		Context:    context.Background(),
		ConfigPath: configPath,
	}
	if err := rootCmd.ExecuteContext(ctx); err != nil {
		os.Exit(1)
	}
}

var rootCmd = &cobra.Command{
	Use:   "designer",
	Short: "designer provides possibility to define new operation and module definitions for orchestrator definition database.",
	PersistentPreRunE: GetStoreFunc,
}

// type holds the definition store which can be used in any cobra command
type DesignerCtx struct {
	context.Context
	Storer model.DefinitionStorer
	Fetcher model.DefinitionFetcher
	ConfigPath string
}

var GetStoreFunc = func(cmd *cobra.Command, args []string) error {
	c, _ := cmd.Context().(*DesignerCtx)
	conf, err := config.GetConfig(c.ConfigPath)
	if err != nil {
		return err
	}
	// initialise definition functions
	conn, err := repository.NewConnection(repository.Options{
		Driver:     conf.Definitions.DBType,
		Host:       conf.Definitions.Options.Host,
		Port:       conf.Definitions.Options.Port,
		Name:       conf.Definitions.Options.Database,
		User:       conf.Definitions.Options.User,
		Password:   conf.Definitions.Options.Password,
	})
	if err != nil {
		return errors.Wrap(err, "Cannot open definition storage:")
	}
	c.Fetcher = repository.NewDefinitionFetcher(conn)
	c.Storer = repository.NewDefinitionStorer(conn)
	return nil
}
