Definícia analytickej operácie
================================

Pojem analytická __operácia__ predstavuje proces vykonania špecifickej analýzy nad objektami dátového skladu. Príkladmi takejto analýzy môže byť vyhľadanie obrázkov s konkrétným vzorom alebo hľadanie komunít v sieťových dátach. Na vykonanie analýz je zvyčajne potreba vykonať viacero krokov (príprava dát, výpočet, spustenie služby grafickeho prostredia pre zhliadnutie výsledkov analýzy,..). Tieto kroky budeme nazývať analytické __moduly__.

Modulom označujeme jeden kontainer (program,skript), ktorý má zvyčajne jedinú úlohu. Berie vstupné parametre (vo forme environment vars), má prístup k objektom dátového skladu a svoje výsledky uloží naspǎť do dátového skladu. Pre lepšiu modularitu nie je modul zviazaný s konkrétnou analytickou operáciu. Môže sa použiť v ktorejkoľvek operácii.

Aby sme boli schopní vytvoriť operáciu ako vykonanie krokov v definovanom poradí, potrebujeme v nej definovať pravidlá, ktoré budú poradie vynucovať. Operácia sa preto skladá z __uzlov__ (kroky) a __hrán__ (pravidlá). Operácia je teda smerový graf. Uzol obsahuje meno definíie modulu, ktorý chceme v kroku použiť. Hrana (A,B) definuje, že modul v uzle B sa vykoná až po úspešnom vykonaní modulu v uzle A.

Definícia modulu
========================

Príprava modulu pozostáva z nasledujúcich krokov:   
1. vytvorenie image modulu
2. nahranie zbuildeného image do repozitára
3. vytvorenie definície modulu
4. vloženie definície do databázy

## vytvorenie image

Na vytvorenie docker image je potrebné naštudovať si [Dockerfile](https://docs.docker.com/engine/reference/builder/).
Tvorca modulu sa môže inšpirovať [Dockerfilom orchestrátora](../Dockerfile).
Platí, ale jedno pravidlo:
Všetky vstupné parametre definované pre modul budú pri vytvorení kontainra vložené ako environment variables. K nim orchestrátor naviac vloží aj ďalšie parametre ako adresu na dátový sklad alebo adresy endpointov ďalších modulov spustených pod rovnakou operáciou. Tabuľka s názvami a popismi týchto parametrov sa nachádza v sekcii __env premenné__ na konci tohto dokumentu.

## nahranie image do repozitára

Pre potreby testovania je pripravený docker repozitár, z ktorého môže naša testovacia platforma (kubernetes) sťahovať požadované image. Repozitár beží pod portom 443 na serveri s názvom "kubernetes-master" v projekte "mu-analyza" na cloude v metacentre. Image môžeme nahrať priamo z tohto serveru alebo aj z lokálneho stroja. Nižšie sú popísane obe varianty:

1. ### Nahratie priamo zo stroja v metacentre:

   Pre pokračovanie je potrebné nastaviť si prístup na server a vytvoriť si svoje pracovné prostredie. Postup nájdete [tu](./prepare_env.md).

   Skopírujeme image z lokálneho stroja na server:
   * Windows: použijeme nástroj <i>Winscp</i>
   * Linux: pomocou utility <i>scp</i>

   Pomocou docker cli nahrajeme image do lokalnej docker cache:
   ```bash
   docker load -i <image>.tar 
   ```
   Následne premenujeme nahraný image tak, aby obsahoval cestu k repozitáru:
   ```bash
   docker tag <názov> orchestrator:443/<názov>:<tag>
   ```
   Teraz sme pripravení na nahratie image do repozitára:
   ```bash
   docker push orchestrator:443/<názov>:<tag>
   ```

2. ### Nahratie image z lokálneho stroja (Advanced)

   Pre vzdialený prístup k repozitári je potrebné:
   * nainštalovať si docker
   * stiahnuť tls certifikát repozitára, ktorý sa nachádza na serveri
   * príhlásiť sa do repozitára s údajmi, ktoré sa nachádzajú na serveri

   Všetky potrebné informácie nájdete na serveri v složke `/home/ubuntu/docker-registry-access`.

   Následné nahratie image vykonáme z lokálneho stroja obdobne ako pri 1. možnosti:

   ```bash
   docker load -i <name>.tar
   docker tag <name> orchestrator:443/<name>:<tag>
   docker push orchestrator:443/<name>:<tag>
   ```

3. ### vytvorenie definície modulu

Aby bol orchestrátor schopný spustiť požadovaný modul, musí mať k dispozícii údaje ako názov imagu predstavujúceho daný modul a ďalšie jeho vlastnosti ako možné vstupné parametre či použité porty. Tieto všetky údaje sú obsiahnuté v jeho definícii, ktorá bude uložená v databáze definícií.

Pre ukladanie definícií modulov sa používa json formát.   
Príklad súboru s definíciou:
```json
{
   "modules": [
      {  
         "name": "modul-test-1",
         "type": "service",
         "description": "testing service",
         "image": "test_1",
         "tag": "1.0",
         "isBlocking": true,
         "healthPort": 6060,
         "ports": [
            {
               "name": "port1",
               "number": 8080,
               "isPublic": true
            }
         ],
         "parameters": [
            {
               "name": "param1",
               "defaultValue": "value1",
               "description": "param1 description",
               "isStrict": true
            }
         ],
         "resources": {
            "cpus": 1,
            "ram": 250 
         }
      }
   ]
}
```

V definícii môže tvorca použiť nasledujúce parametre: 
   * `name` - názov modulu (unikátny, povinný)
   * `description` - voliteľné, popis modulu
   * `type` - typ modulu, môže byť *job* alebo *service*. Job predstavuje úlohu, ktorá eventuálne sama skončí a vyhodnotí sa jej úspech/neúspech. Service slúži na spustenie dlhodobých procesov, ktoré sú nakoniec ukončené manuálne užívateľom skrz niektorý definovaný port alebo zastavením samotnej operácie.
   * `image` - názov image bez tagu v docker repozitári, orchestrátor pri vytváraní kontainra pridá prefix označujúci repozitár
   * `tag` - tag image
   * `isblocking` - udáva, či sa na dokončenie modulu bude čakať alebo sa po úspešnosm štarte štarte prejde na vykonanie ďalších krokov v poradí (defaultne false)
   * `ports` - sieťové porty, na ktorých modul počúva vo forme listu. Port pozostáva z:
     *  `name` - unikátny názov portu
     *  `number` - unikátne číslo portu
     *  `isPublic` - udáva viditeľnosť portu. V prípade hodnotu *false* je port dosieahnuteľný iba z ostatných modulov na platfrome. Pre otvorenie portu pre užívateľov je nutné nastaviť hodnotu na *true* (defaultne false)
   * `healthPort` - voliteľné, označuje číslo portu, na ktorom platforma testuje živosť modulu, modul nebude označený ako *running* pokiaľ požiadavka GET na "/" nevráti code 200.    
   * `parameters` - možné vstupné parametre modulu. Parameter pozostáva z:
      * `name` - názov parametru unikátny pre modul
      * `defaultValue` - defaultná hodnota
      * `description` - voliteľné, popis parametru
      * `isstrict` - chceme aby sa tento parameter bral do úvahy pri kontrole parametrov v prípade možnosti znovu použitia už bežiaceho modulu? Ak áno, nastavíme hodnotu na *true*
   * `resources` - požiadavky na minimálne množstvo vypočtových prostriedkov pre modul
     * `cpus` - počet jadier procesora
     * `ram` - množstvo ram v MiB
   

## vloženie definície do databázy

Pripravený json súbor je možné vložiť do databázy skrz pripravený tool `designer`.

Definícia operácie
========================

Príprava operácie pozostáva z nasledujúcich krokov:   
1. vytvorenie definície operácie
2. vloženie definície do databázy

## Vytvorenie definície operácie

Aby bol orchestrátor schopný spustiť požadovanú operáciu, musí mať k dispozícii jej definíciu, ktorá bude uložená v databáze definícií.   

Pre ukladanie definícií operácií sa používa json formát.   
Príklad súboru s definíciou:
```json
{
   "operations": [
      {
         "name": "test-operation-1",
         "description": "operation description",
         "nodes": [
            {
               "id": 1,
               "description": "description of node",
               "moduleName": "m1",
               "parameters": [
                  {
                     "name": "param1",
                     "DefaultValue": "overriden",
                     "isConstant": true
                  }
               ]
            },
            {
               "id": 2,
               "moduleName": "m2"
            }
         ],
         "edges": [
            {
               "fromNode": 1,
               "toNode": 2
            }   
         ]
      }
   ]
}
```
V definícii môže tvorca použiť nasledujúce parametre: 
   * `name` - unikátny názov operácie
   * `description` - popis operácie
   * `nodes` - list uzlov operácie, predstavujúcich body grafu 
     * `id` - ID uzlu unikátne v rámci operácie, používa sa na referenciu v hranách
     * `description` - popis uzlu
     * `modulename` - meno modulu, ktorý sa použije pri vykonaní uzlu
     * `parameters` - list parametrov, ktorí chceme upraviť pre kontext tejto operácie
       * `name` - meno parametru, ktorý chceme upraviť
       * `defaultValue` - nová defaultná hodnota modulu, ktorá sa použije pri spustení modulu v rámci tohto uzlu 
       * `isConstant` - umožňuje nastaviť parameter ako read-only pre túto operáciu, čo znamená, že užívateľovi nie je dovolené parameter prepísať novou hodnotou pri volaní operácie
   * `edges` - list pravidiel pre poradie vykonávania uzlov, predstavuje hrany grafu
     * `fromNode` - uzol ktorý je spustený pred uzlom uvedenom v toNode
     * `toNode` - uzol, ktorý je spustený až po uzlu uvedenom vo fromNode

## vloženie definície do databázy

Pripravený json súbor je možné vložiť do databázy skrz pripravený tool `designer`.

Poznámky k definíciám
=====================

## definíce v jednom súbore

Definície operácií a modulov môžme uložiť spou do rovnakáho súboru. JSON bude vyzerať takto:
```json
{
   "operations": [
   ],
   "modules": [
   ]
}
```

## env premenné 

Nasledujúce env premenné sa vložia do každého kontainra vytvoreného orchestrátom:

| premenná  | popis  |
| ------- | ------- |
| DATAWAREHOUSE | URL na datový sklad |
| MODULE_ID_SELF | ID bežiaceho modulu, ktoré je zároveň použité ako hostname |
| MODULE_NAME | názov definície bežiaceho modulu |
| PARAMETER_\<MODULE_NAME\>_\<PARAM_NAME\> | hodnota parametru s názvom \<PARAM_NAME\> modulu s názvom defin9cie <MODULE_NAME\> |
| ENDPOINT_\<MODULE_NAME\>_\<PORT_NAME\> | číslo portu, uložené v databáze pod názvom \<PORT_NAME\>, ktoré patrí bežiacemu modulu s názvom jeho definície \<MODULE_NAME\> |
| MODULE_ID_\<NAME\> | ID iného bežiaceho modulu v rámci operácie s názvom definície \<NAME\>, ktoré zároveň slúži ako jeho DNS méno |