# Práca s orchestrátorom a testovanie

__Tento dokument predpokladá pripravené testovacie prostredie a nahraté definície modulov/operácií v databáze.__


Orchestrátor spúšťame príkazom:
```bash
./orchestrator
```
Pre prácu s bežiacim orchestrátorom si otvoríme ďalšie okno príkazovej riadky, Pomocou príkazu `curl` zasielame dotazy na [REST API orchestrátora](./api-doc/swagger.yaml). 

Orchestrátor pri ukončovaní svojho behu ukončí všetky bežiace moduly a operácie a zmaže všetky metadata z databázy. Pre potrebu ponechania behu orchestrátora po dlhšiu dobu môžete využiť utilitu `tmux`. Jej použitie je v ďalších častiach dokumentu.

## Užitočné príkazy:

Spustenie modulu typu <i>job</i> vytvorí v kubernetes objekty [pod](https://kubernetes.io/docs/concepts/workloads/pods/pod/) a [job](https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/). Pre vylistovanie spustených jobov môžeme použiť príkazy:
```bash
kubectl -n <meno našeho namespace> get pods
kubectl -n <meno našeho namespace> get jobs 
```
Pre výpis podrobných informácií o vybranom jobe/pode použijeme:
```bash
kubectl -n <meno našeho namespace> describe pod <meno podu>
kubectl -n <meno našeho namespace> describe job <meno jobu>
```
V podrobnom výpise môžeme nájsť env variables predané orchestrátorom, otvorené porty alebo stav (startovanie/beh/error) samotného kontainra.

Ak sme spustili modul typu <i>service</i>, vytvorili sa objekty pod a [deployment](https://kubernetes.io/docs/concepts/workloads/controllers/deployment/). Pre vylistovanie a podrobný výpis services postupujeme obdobne:
```bash
kubectl -n <meno našeho namespace> get pods
kubectl -n <meno našeho namespace> get deployments 
kubectl -n <meno našeho namespace> describe pod <meno podu>
kubectl -n <meno našeho namespace> describe deployment <meno jobu>
```

Kubernetes umožňuje spustiť príkaz v bežiacom kontainri alebo vypísať jeho stdout/stderr:
Pre vypísanie stdout/err:
```bash
kubectl -n <meno našeho namespace> logs <meno podu> <meno kontainra>
# pod môže obsahovať viacero kontainrov, kubectl podporuje bash completion tak len stlačte tab po napísani mena podu a kontainer vám to doplní
```
Pre spustenie príkazu (napríklad /bin/bash):
```bash
kubectl -n <meno naseho namespace> exec <meno našeho namespace> exec <meno podu> -it -- /bin/bash
```

V prípade nečakanej chyby orchestrátora môže nastať situácia, kedy orchestrator prestane bežať ale kontainre ostanú spustené. Tie je možné zmazať nasledujúcim spôsobom: 
```bash
kubectl -n <meno naseho namespace> delete deployment <nazov deploymentu>
kubectl -n <meno naseho namespace> delete job <nazov jobu>
# nesnažte sa mazať priamo pody
```

V prípade chyby orchestrátora môže taktiež nastať situácia, že sa po skončení behu orchestrátora nezmažú z databázy metadata o bežiacich moduloch. Pred ďalším behom orchestrátora je dobré tieto metadata vymazať.
```bash
# pristup do databazy
psql <meno>
# vypísanie tabulky beziacich modulov
select * from running_modules;
``` 
Ak v databáze metadata vidíme, zmžeme ich so všetkých relevantných tabuliek. Pozri [db schému](https://gitlab.ics.muni.cz/ANALYZA/orchestrator/blob/master/docs/db.png).
```bash
delete from running_modules
```

### tmux

Tmux, terminal multiplexer, umožňuje vytvoriť ďalšie užívateľské sessions v systéme a kedykoľvek sa od nich odpojiť, pričom sa tieto sessions neukončia, ale ostanú bežať na pozadí. Užívateľ sa k nim môže znova pripojiť a pokračovať v práci.

Vytvorenie a pripojenie k novej session:
```bash
tmux new -s <meno session>
```
Odpojenie sa z bežiacej session:
```bash
ctrl+b a následne tlačidlo d
```
Pripojenie sa k bežiacej session:
```bash
tmux a -t <meno session>
```
