// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package app

import (
	"fmt"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/logger"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
)

type Options struct {
	WareHouse  string
	DockerRegistry string
	Definition model.DefinitionFetcher
	Metadata   model.MetadataManager
	Platform   model.PlatformManager
	Logger     logger.Logger
}

type Config struct {
	Warehouse string
	DockerRegistry string
}

// main service
type Orchestrator struct {
	config Config
	logger     logger.Logger
	definition *definitionService
	metadata   *metadataService
	platform   *platformService
	service	   *orchestratorService
}

func NewOrchestrator(c Options) (*Orchestrator, error) {
	if c.Definition == nil {
		return nil, fmt.Errorf("no definition manager specified")
	} else if c.Metadata == nil {
		return nil, fmt.Errorf("no definition manager specified")
	} else if c.Platform == nil {
		return nil, fmt.Errorf("no definition manager specified")
	} else if c.Logger == nil {
		return nil, fmt.Errorf("no definition manager specified")
	}
	app := &Orchestrator{
		config: 	Config{Warehouse: c.WareHouse, DockerRegistry: c.DockerRegistry},
		definition: &definitionService{fetcher: c.Definition, logger: c.Logger},
		metadata:   &metadataService{definitions: c.Definition, repository: c.Metadata, logger: c.Logger},
		platform:   &platformService{manager: c.Platform, logger: c.Logger},
		logger:     c.Logger,
	}
	app.service = newOrchestrationService(app.metadata, app.definition, app.platform, app.logger, app.config)
	return app, nil
}

func (o *Orchestrator) Close() {
	o.service.Close()
	_ = o.metadata.DeleteOperation(&model.Operation{})
	_ = o.metadata.DeleteModule(&model.Module{})
	_ = o.platform.StopModule("")
}