// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package app

import (
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/logger"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
)

// definitionService provides methods for correct population and handling of definition structures
// Definition Fetcher must be provided in order for service to work.
type definitionService struct {
	fetcher model.DefinitionFetcher
	logger logger.Logger
}

// GetOperation returns a number of operations that satisfy provided query.
// Parameter limit sets a maximum number of operations fetched.
// All operations satisfy nil query
func (s definitionService) GetOperation(query *model.OperationDefinition, limit int) ([]*model.OperationDefinition, error) {
	return s.fetcher.QueryOperation(query, limit)
}

// GetNode returns a number of nodes that satisfy provided query.
// Parameter limit sets a maximum number of nodes fetched.
// All nodes satisfy nil query
func (s definitionService) GetNode(query *model.NodeDefinition, limit int) ([]*model.NodeDefinition, error) {
	return s.fetcher.QueryNode(query, limit)
}

// GetEdge returns a number of edges that satisfy provided query.
// Parameter limit sets a maximum number of edges fetched.
// All edges satisfy nil query
func (s definitionService) GetEdge(query *model.EdgeDefinition, limit int) ([]*model.EdgeDefinition, error) {
	return s.fetcher.QueryEdge(query, limit)
}

// GetModule returns a number of modules that satisfy provided query.
// Parameter limit sets a maximum number of modules fetched.
// All modules modules nil query
func (s definitionService) GetModule(query *model.ModuleDefinition, limit int) ([]*model.ModuleDefinition, error) {
	return s.fetcher.QueryModule(query, limit)
}
