// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package config

// Type Config holds all configuration values
type Config struct {
	Definitions db
	Metadata db
	Platform platform
	Api api
	Log log
	Warehouse warehouse
	Docker docker
}

type log struct {
	Debug bool
	Logfile string
}

type warehouse struct {
	Host string
	Port string
}

// Type api represents "api" section of configuration
type api struct {
	Address string
	Port string
}

// Type db represents either metadata or definition sections of configuration
type db struct {
	DBType string
	Options dbOptions
}

// Type docker represents options for docker repository
type docker struct {
	Registry string
}

// Type dbOptions represents subsection of dbConfig
type dbOptions struct {
	Host string
	Port string
	Database string
	User string
	Password string
}

// Type platform represents platform section of configuration
type platform struct {
	PlatformType string
	Options map[string]interface{}
}