// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package logger

import (
	"github.com/sirupsen/logrus"
	"io"
)

type Logger interface {
	Log(level string, args ...interface{})
	GetLogger()*logrus.Logger
	// add functions for specific log formats
}

type logger struct {
	logger *logrus.Logger
}

func (l *logger) GetLogger() *logrus.Logger {
	return l.logger
}

func (l logger) Log(level string, args ...interface{}) {
	lvl, err := logrus.ParseLevel(level)
	if err != nil {
		return
	}
	l.logger.Log(lvl, args)
}

func New(level logrus.Level, format logrus.Formatter, file io.Writer) Logger {
	l := logrus.New()
	l.SetLevel(level)
	l.SetFormatter(format)
	l.SetOutput(file)
	return &logger{
		l,
	}
}
