// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package model

// PlatformManager provides methods to manage running modules on platform
type PlatformManager interface {
	// StartModule takes module metadata, its definition and env Variables to run Module on platform
	StartModule(module *Module, definition *ModuleDefinition, env map[string]string) (*Module, error)
	// StopModule stops and deletes desired module
	StopModule(moduleID string) error
	// StatusModule returns status of running module
	StatusModule(moduleID string) (ModuleStatus, error)
	// Closes connection to platform
	Close() error
}