// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package kubernetes

import (
	appsv1 "k8s.io/api/apps/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// internal helper method
func (m *kubernetesManager) listDeployments() ([]appsv1.Deployment, error) {
	var deploymentList = make([]appsv1.Deployment, 0)
	deps, err := m.c.AppsV1().Deployments(m.config.Namespace).List(metav1.ListOptions{})
	if err != nil {
		return deploymentList, err
	}

	for _, dep := range deps.Items {
		deploymentList = append(deploymentList, dep)
	}
	return deploymentList, nil
}

// internal helper method
func (m *kubernetesManager) getDeployment(name string) (*appsv1.Deployment, error) {
	return m.c.AppsV1().Deployments(m.config.Namespace).Get(name, metav1.GetOptions{})
}

// internal helper method
func (m *kubernetesManager) createDeployment(deployment *appsv1.Deployment) error {
	_, err := m.c.AppsV1().Deployments(m.config.Namespace).Create(deployment)
	return err
}

// internal helper method
func (m *kubernetesManager) updateDeployment(deployment *appsv1.Deployment) error {
	_, err := m.c.AppsV1().Deployments(m.config.Namespace).Update(deployment)
	return err
}

// internal helper method
func (m *kubernetesManager) deleteDeployment(name string) error {
	return m.c.AppsV1().Deployments(m.config.Namespace).Delete(name, &metav1.DeleteOptions{})
}

// internal helper method
func (m *kubernetesManager) deleteAllDeployments() error {
	deps, _ := m.c.AppsV1().Deployments(m.config.Namespace).List(metav1.ListOptions{})
	for _, dep := range deps.Items {
		if err := m.deleteDeployment(dep.Name); err != nil {
			return err
		}
	}
	return nil
}
