// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package kubernetes

import (
	batchv1 "k8s.io/api/batch/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// internal helper method
func (m *kubernetesManager) listJobs() ([]batchv1.Job, error) {
	var jobList = make([]batchv1.Job, 0)
	deps, err := m.c.BatchV1().Jobs(m.config.Namespace).List(metav1.ListOptions{})
	if err != nil {
		return jobList, err
	}

	for _, dep := range deps.Items {
		jobList = append(jobList, dep)
	}

	return jobList, nil
}

// internal helper method
func (m *kubernetesManager) getJob(name string) (*batchv1.Job, error) {
	job, err := m.c.BatchV1().Jobs(m.config.Namespace).Get(name, metav1.GetOptions{})
	return job, err
}

// internal helper method
func (m *kubernetesManager) createJob(job *batchv1.Job) error {
	_, err := m.c.BatchV1().Jobs(m.config.Namespace).Create(job)
	return err
}

// internal helper method
func (m *kubernetesManager) deleteJob(name string) error {
	return m.c.BatchV1().Jobs(m.config.Namespace).Delete(name, &metav1.DeleteOptions{})
}

// internal helper method
func (m *kubernetesManager) deleteAllJobs() error {
	jobs, _ := m.c.BatchV1().Jobs(m.config.Namespace).List(metav1.ListOptions{})
	for _, job := range jobs.Items {
		if err := m.deleteJob(job.Name); err != nil {
			return err
		}
	}
	return nil
}
