// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package kubernetes

import (
	v1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// internal helper method
func (m *kubernetesManager) listPods() (*[]v1.Pod, error) {
	podList := make([]v1.Pod, 0)
	pods, err := m.c.CoreV1().Pods(m.config.Namespace).List(metav1.ListOptions{})
	if err != nil {
		return &podList, err
	}

	for _, pod := range pods.Items {
		podList = append(podList, pod)
	}

	return &podList, nil
}

// internal helper method
func (m *kubernetesManager) getPod(name string) (*v1.Pod, error) {
	return m.c.CoreV1().Pods(m.config.Namespace).Get(name, metav1.GetOptions{})
}

// internal helper method
func (m *kubernetesManager) createPod(pod *v1.Pod) (*v1.Pod, error) {
	return m.c.CoreV1().Pods(m.config.Namespace).Create(pod)
}

// internal helper method
func (m *kubernetesManager) deletePod(name string) error {
	return m.c.CoreV1().Pods(m.config.Namespace).Delete(name, &metav1.DeleteOptions{})
}
