// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package kubernetes

import (
	v1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// internal helper method
func (m *kubernetesManager) listServices() ([]v1.Service, error) {
	serviceList := make([]v1.Service, 0)
	services, err := m.c.CoreV1().Services(m.config.Namespace).List(metav1.ListOptions{})
	if err != nil {
		return serviceList, err
	}

	for _, svc := range services.Items {
		serviceList = append(serviceList, svc)
	}

	return serviceList, nil
}

// internal helper method
func (m *kubernetesManager) getService(name string) (*v1.Service, error) {
	return m.c.CoreV1().Services(m.config.Namespace).Get(name, metav1.GetOptions{})
}

// internal helper method
func (m *kubernetesManager) createService(service *v1.Service) (*v1.Service, error) {
	p, err := m.c.CoreV1().Services(m.config.Namespace).Create(service)
	return p, err
}

// internal helper method
func (m *kubernetesManager) updateService(service *v1.Service) (*v1.Service, error) {
	p, err := m.c.CoreV1().Services(m.config.Namespace).Update(service)
	return p, err
}

// internal helper method
func (m *kubernetesManager) deleteService(name string) error {
	return m.c.CoreV1().Services(m.config.Namespace).Delete(name, &metav1.DeleteOptions{})
}
