// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package repository

import (
	"fmt"
	_ "github.com/jinzhu/gorm/dialects/sqlite"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
	"testing"
)

var ExampleOperation1 = model.OperationDefinition{
	Name:  "exampleOperation1",
	Nodes: []model.NodeDefinition{
		{
			ID:            1,
			ModuleName:    "module1",
			OperationName: "",
			Parameters:    []model.NodeDefinitionParameter{
				{
					NodeID:       0,
					Name:         "param1",
					DefaultValue: nil,
					IsConstant:   false,
				},
			},
		},
		{
			ID:            2,
			ModuleName:    "module2",
			OperationName: "",
			Parameters:    []model.NodeDefinitionParameter{
				{
					NodeID:       0,
					Name:         "param1",
					DefaultValue: nil,
					IsConstant:   false,
				},
			},
			OperationRef:  nil,
		},
	},
	Edges: []model.EdgeDefinition{
		{
			OperationName: "",
			FromNode:      1,
			ToNode:        2,
		},
	},
}

var exampleModule1 = model.ModuleDefinition{
	Name:       "module1",
	Type:       "job",
	IsBlocking: false,
	HealthPort: 0,
	Image:      "test1",
	Tag:        "1.0",
	Parameters: []model.ModuleDefinitionParameter{
		{
			Module:       "",
			Name:         "param1",
			Description:  "description1",
			DefaultValue: "value1",
			IsStrict:     true,
		},
	},
	Ports: []model.ModuleDefinitionPort{
		{
			Module:   "",
			Name:     "port1",
			Number:   80,
			IsPublic: false,
		},
	},
}

var exampleModule2 = model.ModuleDefinition{
	Name:       "module2",
	Type:       "service",
	IsBlocking: false,
	HealthPort: 0,
	Image:      "test2",
	Tag:        "1.0",
	Parameters: []model.ModuleDefinitionParameter{
		{
			Module:       "",
			Name:         "param1",
			Description:  "description1",
			DefaultValue: "value1",
			IsStrict:     true,
		},
	},
	Ports: []model.ModuleDefinitionPort{
		{
			Module:   "",
			Name:     "port1",
			Number:   80,
			IsPublic: false,
		},
	},
}

func TestDefinition(t *testing.T) {
	var err error
	store, _ := NewDefinitionRepository(DbConfig{Driver: "sqlite3"})
	operations, err := store.QueryOperation(nil,1)
	if err != nil {
		fmt.Println("b")
		panic(err)
	}
	fmt.Println(operations)
	err = store.DeleteOperation(operations[0])
	nodes, err := store.QueryNode(nil, 0)
	if err != nil {
		fmt.Println("a")
		panic(err)
	}
	fmt.Println(nodes)
	modules, err := store.QueryModule(nil,0)
	if err != nil {
		fmt.Println("c")
		panic(err)
	}
	fmt.Println(modules[0])
}
