// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package repository

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"gitlab.ics.muni.cz/ANALYZA/orchestrator/internal/model"
	"testing"
)

var OperationMetadata1 = model.Operation{
	ID:              "op1",
	Definition:      "def1",
	InputParameters: []model.OperationParameter{
		{
			ModuleName:  "module1",
			Key:         "key1",
			Value:       "value1",
		},
	},
	Status: model.OperationStatusScheduling,
	Nodes:           []model.Node{
		{
			ID:            "N1",
			ModuleID:      func(i string) *string { return &i }("M1"),
			Definition:    5,
		},
		{
			ID:            "N2",
			ModuleID:      func(i string) *string { return &i }("M2"),
			Definition:    5,
		},
	},
	Edges:           []model.Edge{
		{
			FromNode:    "N1",
			ToNode:      "N2",
		},
	},
}

var ModuleMetadata1 = model.Module{
	ID:         "M1",
	Status:     model.ModuleStatusCreating,
	Definition: "definition1",
	InNodes:    nil,
	Parameters:    []model.ModuleParameter{
		{
			Key:       "param1",
			Value:     "value1",
		},
	},
	Ports:         []model.ModulePort{
		{
			Name:         "port1",
			Number:       80,
			PublicNumber: 0,
			IsPublic:     false,
		},
	},
}

func TestMetadata(t *testing.T) {
	db, err := gorm.Open("sqlite3", "file::memory:?cache=shared")
	if err != nil {
		return
	}
	metaStore, _ := NewMetadataRepository(DbConfig{Connection: db})
	_ = metaStore.StoreModule(&ModuleMetadata1)
	_ = metaStore.StoreOperation(&OperationMetadata1)
	modules, err := metaStore.QueryModule(nil, 0)
	if err != nil {
		fmt.Println("a")
		return
	}
	fmt.Println(modules)
}