// Copyright (C) 2020 Masaryk University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package mEngine

import (
	"fmt"
	"strconv"
	"testing"
	"time"
)

// TODO write tests

func Test1(t *testing.T) {
	e := NewEngine(FlushPeriod(time.Millisecond * 500))
	values := make([]int, 1000000)
	n := time.Now()
	for i := 0; i < 1000000; i++ {
		i := i
		Runner1 := func(doneFunc DoneFunc) {
			defer close(doneFunc())
			values[i] = 0
			<-time.After(time.Second * 1)
		}
		Runner2 := func(doneFunc DoneFunc) {
			defer close(doneFunc())
			values[i] += 5
			<-time.After(time.Second * 1)
		}
		Runner3 := func(doneFunc DoneFunc) {
			defer close(doneFunc())
			values[i] += 25
			<-time.After(time.Second * 4)
		}
		Blocker := func(stopFunc StopFunc, doneFunc DoneFunc) {
			var x = 0
			defer close(doneFunc())
			for {
				if x == 5 {
					return
				}
				select {
				case <- stopFunc():
					return
				default:
					<- time.After(time.Second * 1)
					values[i] += 1
					x += 1
				}
			}
		}
		e.NewTaskWithCancel(strconv.Itoa(i), Runner1, Blocker, Blocker)
		e.NewTask(strconv.Itoa(i), Runner2, Blocker)
		e.NewTaskWithCancel(strconv.Itoa(i), Runner3)
	}

	fmt.Println("Closing")
	fmt.Println(time.Since(n))
	e.Close()
	fmt.Println(values)
	fmt.Println(time.Since(n))
}

func Test2(t *testing.T) {
	run1 := false
	Runner1 := func(doneFunc DoneFunc) {
		defer close(doneFunc())
		<-time.After(time.Second * 1)
		fmt.Println("runer 1")
		run1 = true
	}
	Runner2 := func(doneFunc DoneFunc) {
		defer close(doneFunc())
		<-time.After(time.Second * 1)
		fmt.Println("runer 2")
	}
	Blocker1 := func(stopFunc StopFunc, doneFunc DoneFunc) {
		defer close(doneFunc())
		t := time.After(time.Second * 5)
		for {
			select {
			case <- stopFunc():
				fmt.Println("blocker 1")
				return
			case <- t:
				fmt.Println("blocker 1")
				return
			default:
			}
		}

	}
	Blocker3 := func(stopFunc StopFunc, doneFunc DoneFunc) {
		defer close(doneFunc())
		t := time.After(time.Second * 6)
		for {
			select {
			case <- stopFunc():
				fmt.Println("blocker 3")
				return
			case <- t:
				fmt.Println("blocker 3")
				return
			default:
			}
		}

	}
	Blocker2 := func(stopFunc StopFunc, doneFunc DoneFunc) {
		defer close(doneFunc())
		for {
			if run1 {
				fmt.Println("blocker 2")
				return
			}
			select {
			case <- stopFunc():
				fmt.Println("blocker 2")
				return
			default:
			}
		}
	}
	e := NewEngine(FlushPeriod(time.Millisecond * 500))
	e.NewTask("1", Runner1, Blocker1, Blocker3)
	e.NewTask("2", Runner2, Blocker2)
	time.Sleep(time.Second * 10)
	e.Close()
}
